@@ -244,8 +244,8 @@ DEFINE_SET_PAIR_SCALE_VALUE(palm_dimensions,
                             "palm_detection_depth_scale",
                             GPDS_TOUCHPAD_PALM_DIMENSIONS)
 DEFINE_SET_PAIR_SCALE_VALUE(scrolling_distance,
-                            "horizontal_scrolling_scale",
                             "vertical_scrolling_scale",
+                            "horizontal_scrolling_scale",
                             GPDS_TOUCHPAD_SCROLLING_DISTANCE)
 
 static void
@@ -446,44 +446,83 @@ set_sensitivity_depends_on_use_type (GpdsUI *ui,
 
     switch (use_type) {
     case GPDS_TOUCHPAD_USE_TYPE_OFF:
-    case GPDS_TOUCHPAD_USE_TYPE_TAPPING_AND_SCROLLING_OFF:
+        set_widget_sensitivity(builder, "general_box", FALSE);
         set_widget_sensitivity(builder, "scrolling_vbox", FALSE);
         set_widget_sensitivity(builder, "tapping_vbox", FALSE);
+        set_widget_sensitivity(builder, "speed_vbox", FALSE);
+        break;
+    case GPDS_TOUCHPAD_USE_TYPE_TAPPING_AND_SCROLLING_OFF:
+        set_widget_sensitivity(builder, "general_box", TRUE);
+        set_widget_sensitivity(builder, "scrolling_vbox", FALSE);
+        set_widget_sensitivity(builder, "tapping_vbox", FALSE);
+        set_widget_sensitivity(builder, "speed_vbox", TRUE);
         break;
     case GPDS_TOUCHPAD_USE_TYPE_NORMAL:
+        set_widget_sensitivity(builder, "general_box", TRUE);
         set_widget_sensitivity(builder, "scrolling_vbox", TRUE);
         set_widget_sensitivity(builder, "tapping_vbox", TRUE);
+        set_widget_sensitivity(builder, "speed_vbox", TRUE);
     default:
         break;
     }
 }
 
 static void
-set_touchpad_use_type_combo_state (GpdsUI *ui, GpdsTouchpadUseType use_type)
+set_touchpad_use_type (GpdsUI *ui, GpdsTouchpadUseType use_type)
 {
-    GtkComboBox *combo;
+    GtkToggleButton *button;
     GtkBuilder *builder;
+    gboolean disable_touchpad, disable_tapping_and_scrolling;
+
+    disable_touchpad = (use_type == GPDS_TOUCHPAD_USE_TYPE_OFF);
+    disable_tapping_and_scrolling = (use_type == GPDS_TOUCHPAD_USE_TYPE_TAPPING_AND_SCROLLING_OFF);
 
     builder = gpds_ui_get_builder(ui);
+    button = GTK_TOGGLE_BUTTON(gtk_builder_get_object(builder, "disable_touchpad"));
+    gtk_toggle_button_set_active(button, disable_touchpad);
+
+    button = GTK_TOGGLE_BUTTON(gtk_builder_get_object(builder, "disable_tapping_and_scrolling"));
+    gtk_toggle_button_set_active(button, disable_tapping_and_scrolling);
 
-    combo = GTK_COMBO_BOX(gtk_builder_get_object(builder, "touchpad_use_type"));
-    gtk_combo_box_set_active(combo, (gint)use_type);
     set_sensitivity_depends_on_use_type(ui, use_type);
 }
 
 static void
-cb_touchpad_use_type_changed (GtkComboBox *combo, gpointer user_data)
+cb_touchpad_use_type_toggled (GtkToggleButton *button, gpointer user_data)
 {
     GpdsUI *ui = GPDS_UI(user_data);
+    GpdsTouchpadUseType use_type;
     gint properties[1];
     GError *error = NULL;
+    GtkToggleButton *disable_touchpad_button;
+    GtkToggleButton *disable_tapping_and_scrolling_button;
+    GtkBuilder *builder;
+    gboolean disable_touchpad, disable_tapping_and_scrolling;
     GpdsXInput *xinput;
 
     xinput = gpds_xinput_ui_get_xinput(GPDS_XINPUT_UI(ui));
     if (!xinput)
         return;
 
-    properties[0] = gtk_combo_box_get_active(combo);
+    builder = gpds_ui_get_builder(ui);
+    disable_touchpad_button =
+        GTK_TOGGLE_BUTTON(gtk_builder_get_object(builder, "disable_touchpad"));
+    disable_tapping_and_scrolling_button =
+        GTK_TOGGLE_BUTTON(gtk_builder_get_object(builder, "disable_tapping_and_scrolling"));
+
+    disable_touchpad = gtk_toggle_button_get_active(disable_touchpad_button);
+    disable_tapping_and_scrolling =
+        gtk_toggle_button_get_active(disable_tapping_and_scrolling_button);
+
+    if (disable_touchpad)
+        use_type = GPDS_TOUCHPAD_USE_TYPE_OFF;
+    else {
+        use_type = disable_tapping_and_scrolling ?
+            GPDS_TOUCHPAD_USE_TYPE_TAPPING_AND_SCROLLING_OFF :
+            GPDS_TOUCHPAD_USE_TYPE_NORMAL;
+    }
+
+    properties[0] = use_type;
     if (!gpds_xinput_set_int_properties(xinput,
                                         GPDS_TOUCHPAD_OFF,
                                         &error,
@@ -494,8 +533,103 @@ cb_touchpad_use_type_changed (GtkComboBox *combo, gpointer user_data)
             g_error_free(error);
         }
     }
-    gpds_ui_set_gconf_int(ui, GPDS_TOUCHPAD_OFF_KEY, properties[0]);
-    set_sensitivity_depends_on_use_type(ui, properties[0]);
+
+    gpds_ui_set_gconf_int(ui, GPDS_TOUCHPAD_OFF_KEY, (gint)use_type);
+    set_sensitivity_depends_on_use_type(ui, use_type);
+}
+
+static void
+cb_disable_touchpad_toggled (GtkToggleButton *button, gpointer user_data)
+{
+    cb_touchpad_use_type_toggled(button, user_data);
+}
+
+static void
+cb_disable_tapping_and_scrolling_toggled (GtkToggleButton *button, gpointer user_data)
+{
+    cb_touchpad_use_type_toggled(button, user_data);
+}
+
+static void
+cb_disable_while_other_device_exists_toggled (GtkToggleButton *button, gpointer user_data)
+{
+    GpdsUI *ui = GPDS_UI(user_data);
+
+    gpds_ui_set_gconf_bool(ui,
+                           GPDS_TOUCHPAD_DISABLE_WHILE_OTHER_DEVICE_EXISTS_KEY,
+                           gtk_toggle_button_get_active(button));
+}
+
+static void
+set_move_speed_property (GpdsXInput *xinput, GtkBuilder *builder)
+{
+    GError *error = NULL;
+    GObject *object;
+    gdouble properties[4];
+
+    object = gtk_builder_get_object(builder, "minimum_speed_scale");
+    properties[0] = gtk_range_get_value(GTK_RANGE(object));
+
+    object = gtk_builder_get_object(builder, "maximum_speed_scale");
+    properties[1] = gtk_range_get_value(GTK_RANGE(object));
+
+    object = gtk_builder_get_object(builder, "acceleration_factor_scale");
+    properties[2] = gtk_range_get_value(GTK_RANGE(object));
+
+    if (!gpds_xinput_set_float_properties(xinput,
+                                          GPDS_TOUCHPAD_MOVE_SPEED,
+                                          &error,
+                                          properties,
+                                          4)) {
+        if (error) {
+            show_error(error);
+            g_error_free(error);
+        }
+    }
+}
+
+static void
+cb_move_speed_scale_value_changed (GtkRange *range, gpointer user_data)
+{
+    GtkBuilder *builder;
+    GpdsXInput *xinput;
+
+    xinput = gpds_xinput_ui_get_xinput(GPDS_XINPUT_UI(user_data));
+    if (!xinput)
+        return;
+
+    builder = gpds_ui_get_builder(GPDS_UI(user_data));
+    set_move_speed_property(xinput, builder);
+}
+
+static void
+cb_minimum_speed_scale_value_changed (GtkRange *range, gpointer user_data)
+{
+    gdouble value;
+    cb_move_speed_scale_value_changed(range, user_data);
+
+    value = gtk_range_get_value(range);
+    gpds_ui_set_gconf_float(GPDS_UI(user_data), GPDS_TOUCHPAD_MINIMUM_SPEED_KEY, value);
+}
+
+static void
+cb_maximum_speed_scale_value_changed (GtkRange *range, gpointer user_data)
+{
+    gdouble value;
+    cb_move_speed_scale_value_changed(range, user_data);
+
+    value = gtk_range_get_value(range);
+    gpds_ui_set_gconf_float(GPDS_UI(user_data), GPDS_TOUCHPAD_MAXIMUM_SPEED_KEY, value);
+}
+
+static void
+cb_acceleration_factor_scale_value_changed (GtkRange *range, gpointer user_data)
+{
+    gdouble value;
+    cb_move_speed_scale_value_changed(range, user_data);
+
+    value = gtk_range_get_value(range);
+    gpds_ui_set_gconf_float(GPDS_UI(user_data), GPDS_TOUCHPAD_ACCELERATION_FACTOR_KEY, value);
 }
 
 static void
@@ -509,7 +643,9 @@ setup_signals (GpdsUI *ui, GtkBuilder *builder)
                      G_CALLBACK(cb_ ## object_name ## _ ## signal_name),\
                      ui)
 
-    CONNECT(touchpad_use_type, changed);
+    CONNECT(disable_touchpad, toggled);
+    CONNECT(disable_tapping_and_scrolling, toggled);
+    CONNECT(disable_while_other_device_exists, toggled);
     CONNECT(guest_mouse_off, toggled);
     CONNECT(palm_detection, toggled);
     CONNECT(palm_detection_width_scale, value_changed);
@@ -528,6 +664,10 @@ setup_signals (GpdsUI *ui, GtkBuilder *builder)
     CONNECT(two_finger_vertical_scrolling, toggled);
     CONNECT(two_finger_horizontal_scrolling, toggled);
 
+    CONNECT(minimum_speed_scale, value_changed);
+    CONNECT(maximum_speed_scale, value_changed);
+    CONNECT(acceleration_factor_scale, value_changed);
+
     /* cirlular scrolling trigger */
     CONNECT(trigger_top_toggle, button_press_event);
     CONNECT(trigger_top_right_toggle, button_press_event);
@@ -691,7 +831,7 @@ set_touchpad_use_type_property_from_preference (GpdsUI *ui)
 
     if (!gpds_ui_get_gconf_int(ui, GPDS_TOUCHPAD_OFF_KEY, (gint*)&type))
         type = values[0];
-    set_touchpad_use_type_combo_state(ui, type);
+    set_touchpad_use_type(ui, type);
 
     g_free(values);
 }
@@ -719,6 +859,36 @@ set_click_action (GpdsUI *ui)
     g_free(values);
 }
 
+static void
+set_move_speed_properties_from_preference (GpdsUI *ui, GtkBuilder *builder)
+{
+    GObject *object;
+    gdouble *values;
+    gulong n_values;
+    gdouble value;
+
+    if (!gpds_xinput_ui_get_xinput_float_property(GPDS_XINPUT_UI(ui),
+                                                  GPDS_TOUCHPAD_MOVE_SPEED,
+                                                  &values, &n_values)) {
+        return;
+    }
+
+    if (!gpds_ui_get_gconf_float(ui, GPDS_TOUCHPAD_MINIMUM_SPEED_KEY, &value))
+        value = values[0];
+    object = gtk_builder_get_object(builder, "minimum_speed_scale");
+    gtk_range_set_value(GTK_RANGE(object), value);
+    if (!gpds_ui_get_gconf_float(ui, GPDS_TOUCHPAD_MAXIMUM_SPEED_KEY, &value))
+        value = values[1];
+    object = gtk_builder_get_object(builder, "maximum_speed_scale");
+    gtk_range_set_value(GTK_RANGE(object), value);
+    if (!gpds_ui_get_gconf_float(ui, GPDS_TOUCHPAD_ACCELERATION_FACTOR_KEY, &value))
+        value = values[2];
+    object = gtk_builder_get_object(builder, "acceleration_factor_scale");
+    gtk_range_set_value(GTK_RANGE(object), value);
+
+    g_free(values);
+}
+
 static void
 setup_current_values (GpdsUI *ui, GtkBuilder *builder)
 {
@@ -752,6 +922,7 @@ setup_current_values (GpdsUI *ui, GtkBuilder *builder)
     set_two_finger_scrolling_property_from_preference(ui, builder);
     set_touchpad_use_type_property_from_preference(ui);
     set_circular_scrolling_trigger_property_from_preference(ui);
+    set_move_speed_properties_from_preference(ui, builder);
     set_click_action(ui);
 }
 
